# bijoe - BI dashboard
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
from jsonfield import JSONField


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)


class Visualization(models.Model):
    name = models.TextField(verbose_name=_('name'))
    parameters = JSONField(verbose_name=_('parameters'), encoder_class=JSONEncoder)

    class Meta:
        ordering = ('name', 'id')
        verbose_name = _('visualization')
        verbose_name_plural = _('visualizations')

    def __unicode__(self):
        return self.name
